package com.lzmispt.data.resource.service.history;

package .tmp;

import com.alibaba.fastjson.JSON;
import com.lzmispt.data.center.entity.dto.databasestruct.ColumnParamDTO;
import com.lzmispt.data.center.entity.dto.databasestruct.DBColumnDTO;
import com.lzmispt.data.center.entity.dto.databasestruct.TableParamDTO;
import com.lzmispt.data.center.entity.dto.log.CommonLogDTO;
import com.lzmispt.data.center.entity.enums.BaseSysFieldEnum;
import com.lzmispt.data.center.entity.enums.FileSysFieldEnum;
import com.lzmispt.data.center.entity.enums.StatusCodeEnum;
import com.lzmispt.data.center.entity.vo.standard.metadata.MetadataEntityDataItemVO;
import com.lzmispt.data.center.entity.vo.standard.metadata.MetadataTypeVO;
import com.lzmispt.data.resource.client.DataStandardClient;
import com.lzmispt.data.resource.entity.dto.metadata.TableStructInfoDTO;
import com.lzmispt.data.resource.entity.dto.resanddata.ResAndDataFileFieldDTO;
import com.lzmispt.data.resource.entity.po.file.FileInfoFieldPO;
import com.lzmispt.data.resource.entity.po.metadata.MetadataEntityInfoItemPO;
import com.lzmispt.data.resource.manager.file.FileInfoManager;
import com.lzmispt.data.resource.manager.metadata.MetadataManager;
import com.lzmispt.data.resource.service.metadata.IInfoItem;
import com.lzmispt.data.resource.service.metadata.IInfoItemSync;
import com.lzmispt.dcbase.ServiceUtil;
import com.lzmispt.dcbase.databasestruct.DataStructProvider;
import com.lzmispt.dcbase.databasestruct.TypeConverter;
import com.lzmispt.dcbase.dcconst.DCConst;
import com.lzmispt.dcbase.historytable.HistoryTableConfig;
import com.lzmispt.dcbase.sysfield.SysFieldHelper;
import com.lzmispt.dcbase.util.HeaderInterceptor;
import com.lzmispt.framework.entity.entity.ResultEntity;
import com.lzmispt.framework.exception.CustomException;
import com.lzmispt.framework.exception.ExceptionUtil;
import com.lzmispt.framework.fconst.FrameConst;
import com.lzmispt.framework.utils.RabbitMQUtil;
import com.lzmispt.framework.utils.WorkerId;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.stereotype.Service;

@Service
@RefreshScope
public class HistoryTableManager {
  private static final String HISTORY_TABLE_SUFFIX = "_history";
  
  private static final int INTEGER_DEFAULT_LENGTH = 4;
  
  private static final String PARAM_TAG_PRE = "#{";
  
  private static final String PARAM_EQUAL_TAG_PRE = "=#{";
  
  private static final String AND_TAG = " and ";
  
  private static final String LZDC_SYS_VERSION = "lzdc_sys_version";
  
  private static final String LZDC_SYS_END_TIME = "lzdc_sys_end_time";
  
  public static final List<String> HIS_TABLE_FIELD = Collections.unmodifiableList(Arrays.asList(new String[] { "lzdc_sys_end_time", "lzdc_sys_version" }));
  
  private static final String DEFAULT_END_TIME = "9999-12-31 23:59:59";
  
  @Autowired
  private HistoryTableConfig historyTableConfig;
  
  @Resource
  private DataStandardClient dataStandardClient;
  
  public Boolean getEnableHistory(String platCode, String tableType) {
    HistoryTableConfig.TypeEnable platEnable;
    HistoryTableConfig.Enable enable;
    if ("Apply".equals(platCode)) {
      platEnable = this.historyTableConfig.getApply();
    } else {
      platEnable = "Manage".equals(platCode) ? this.historyTableConfig.getManage() : this.historyTableConfig.getCollection();
    } 
    if (DCConst.TableType.DATATABLE.equals(tableType)) {
      enable = platEnable.getDataTable();
    } else if (DCConst.TableType.FILETABLE.equals(tableType)) {
      enable = platEnable.getFileTable();
    } else if (DCConst.TableType.SPATIAL_TABLE.equals(tableType)) {
      enable = platEnable.getSpatialTable();
    } else {
      return Boolean.valueOf(false);
    } 
    return Boolean.valueOf((Boolean.TRUE.equals(enable.getEnableHistory()) && Boolean.TRUE.equals(enable.getHistoryCheck())));
  }
  
  public ResultEntity<Boolean> checkHistoryTableName(DataStructProvider dataStructProvider, String originTableName, String platCode, String tableType) {
    return checkHistoryTableName(dataStructProvider, originTableName, false, platCode, tableType);
  }
  
  public ResultEntity<Boolean> checkHistoryTableName(DataStructProvider dataStructProvider, String originTableName, boolean isImport, String platCode, String tableType) {
    if (!getEnableHistory(platCode, tableType).booleanValue() || dataStructProvider == null || originTableName == null)
      return ResultEntity.successMessage(Boolean.valueOf(true)); 
    if (originTableName.toLowerCase().endsWith("_history"))
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_TABLE_NAME_IS_HISTORY_TABLE_PATTERN.getCode()), StatusCodeEnum.DATA_RESOURCE_TABLE_NAME_IS_HISTORY_TABLE_PATTERN
          .getMessage()); 
    String historyTableName = getHistoryTableName(originTableName);
    Boolean existsTable = dataStructProvider.isExistsTable(historyTableName);
    if (existsTable == null || !existsTable.booleanValue())
      return ResultEntity.successMessage(Boolean.valueOf(true)); 
    if (isImport) {
      int tableDataCount = dataStructProvider.getTableDataCount(historyTableName);
      if (tableDataCount > 0)
        return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_HISTORY_TABLE_NAME_IN_USE_HAS_DATA.getCode()), 
            String.format(StatusCodeEnum.DATA_RESOURCE_HISTORY_TABLE_NAME_IN_USE_HAS_DATA.getMessage(), new Object[] { originTableName })); 
      return ResultEntity.successMessage(Boolean.valueOf(true));
    } 
    return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_HISTORY_TABLE_NAME_IN_USE.getCode()), StatusCodeEnum.DATA_RESOURCE_HISTORY_TABLE_NAME_IN_USE
        .getMessage());
  }
  
  public void saveHistoryTable(DataStructProvider dataStructProvider, Long entityId, String originTableName, String platCode, String tokenId, Integer isRes) {
    saveHistoryTable(dataStructProvider, entityId, null, originTableName, platCode, tokenId, DCConst.TableType.DATATABLE, isRes);
  }
  
  public void saveHistoryTable(DataStructProvider dataStructProvider, Long entityId, List<Long> infoItemIds, String originTableName, String platCode, String tokenId, String tableType, Integer isRes) {
    if (!getEnableHistory(platCode, tableType).booleanValue())
      return; 
    Map<String, DBColumnDTO> originTableColumnMap = (Map<String, DBColumnDTO>)dataStructProvider.getEntityDbTableStruct(originTableName, null).stream().filter(item -> !SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.LAST_MODIFY_TIME).equalsIgnoreCase(item.getColumnName())).collect(Collectors.toMap(item -> item.getColumnName().toLowerCase(), Function.identity(), (k1, k2) -> k1));
    List<MetadataEntityInfoItemPO> syncInfoItems = (List<MetadataEntityInfoItemPO>)MetadataManager.getEntityInfoItemsByEntityId(entityId, isRes, platCode, tokenId).stream().filter(item -> originTableColumnMap.containsKey(item.getDataItemName().toLowerCase())).filter(item -> (infoItemIds == null || infoItemIds.contains(item.getInfoItemId()) || DCConst.TRUE.equals(item.getInfoItemStatus()) || SysFieldHelper.sysFieldContains(item.getDataItemName()))).collect(Collectors.toList());
    Set<Long> dataItemIdSet = (Set<Long>)syncInfoItems.stream().map(MetadataEntityInfoItemPO::getDataItemId).collect(Collectors.toSet());
    Map<String, MetadataEntityDataItemVO> dataItemMap = new HashMap<>();
    ResultEntity<List<MetadataEntityDataItemVO>> dataItemsResult = this.dataStandardClient.getMetadataEntityDataItemByIds(new ArrayList<>(dataItemIdSet), platCode, tokenId);
    if (dataItemsResult != null && dataItemsResult.getObj() != null) {
      Map<Long, MetadataEntityDataItemVO> collect = (Map<Long, MetadataEntityDataItemVO>)((List)dataItemsResult.getObj()).stream().collect(Collectors.toMap(MetadataEntityDataItemVO::getDataItemId, Function.identity()));
      dataItemMap = (Map<String, MetadataEntityDataItemVO>)syncInfoItems.stream().collect(Collectors.toMap(item -> item.getDataItemName().toLowerCase(), item -> (MetadataEntityDataItemVO)collect.get(item.getDataItemId()), (k1, k2) -> k1));
    } 
    boolean isPass = checkAndCreateHistoryTable(dataStructProvider, originTableName, syncInfoItems, dataItemMap, platCode, tokenId, DCConst.TableType.DATATABLE);
    if (!isPass)
      return; 
    String historyTableName = getHistoryTableName(originTableName);
    Map<String, DBColumnDTO> historyTableColumnMap = (Map<String, DBColumnDTO>)dataStructProvider.getEntityDbTableStruct(historyTableName, null).stream().collect(Collectors.toMap(item -> item.getColumnName().toLowerCase(), Function.identity(), (k1, k2) -> k1));
    for (DBColumnDTO originTableColumn : originTableColumnMap.values()) {
      String lowerCaseColumnName = originTableColumn.getColumnName().toLowerCase();
      MetadataEntityDataItemVO dataItem = dataItemMap.get(lowerCaseColumnName);
      if (dataItem == null)
        continue; 
      DBColumnDTO historyTableColumn = historyTableColumnMap.get(lowerCaseColumnName);
      try {
        if (historyTableColumn != null) {
          if (!Objects.equals(originTableColumn.getDataType(), historyTableColumn.getDataType()) || 
            !Objects.equals(originTableColumn.getDataLength(), historyTableColumn.getDataLength()) || 
            !Objects.equals(originTableColumn.getScale(), historyTableColumn.getScale()) || 
            !Objects.equals(originTableColumn.getIsNullAble(), historyTableColumn.getIsNullAble())) {
            boolean bool = alterColumn(dataStructProvider, historyTableName, originTableColumn, dataItem, tokenId, platCode);
            if (!bool)
              logErrorInfo(null, getColumnLogMsg(originTableName, "修改", originTableColumn.getColumnName()), platCode, tokenId); 
          } 
          continue;
        } 
        boolean success = dataStructProvider.createColumn(historyTableName, originTableColumn.getColumnName(), (
            (IInfoItemSync)ServiceUtil.getInfoItemSyncManager(platCode, IInfoItemSync.class)).getDBColumnType(dataStructProvider.getDbType(), dataItem.getDataType(), platCode, tokenId), dataItem
            .getDataLength(), dataItem.getDataScale(), DCConst.FALSE.equals(originTableColumn.getIsNullAble())).booleanValue();
        if (!success)
          logErrorInfo(null, getColumnLogMsg(originTableName, "添加", originTableColumn.getColumnName()), platCode, tokenId); 
      } catch (Exception e) {
        logErrorInfo(ExceptionUtil.getStackTraceMsg(e), getColumnLogMsg(originTableName, "处理", originTableColumn.getColumnName()), platCode, tokenId);
      } 
    } 
    DBColumnDTO dbColumnDTO = historyTableColumnMap.get("lzdc_sys_end_time");
    checkEndTimeColumn(dataStructProvider, historyTableName, dbColumnDTO);
    TableStructInfoDTO tableStructInfoDTO = getTableStructInfo(dataStructProvider, historyTableName);
    List<MetadataEntityInfoItemPO> needSetDesc = (List<MetadataEntityInfoItemPO>)syncInfoItems.stream().filter(item -> !SysFieldHelper.sysFieldContains(item.getDataItemName())).collect(Collectors.toList());
    ((IInfoItem)ServiceUtil.getInfoItemManager(platCode, IInfoItem.class)).setDBFieldDesc(tableStructInfoDTO, needSetDesc, dataStructProvider, historyTableName, platCode, tokenId);
  }
  
  private void createHistoryTable(DataStructProvider dataStructProvider, String originTableName, List<MetadataEntityInfoItemPO> syncInfoItems, Map<String, MetadataEntityDataItemVO> dataItemMap, String platCode, String tokenId, String tableType) {
    String historyTableName = getHistoryTableName(originTableName);
    IInfoItem infoItemManager = (IInfoItem)ServiceUtil.getInfoItemManager(platCode, IInfoItem.class);
    TableParamDTO table = new TableParamDTO();
    List<ColumnParamDTO> columns = new ArrayList<>();
    String lastModifyTime = DCConst.TableType.DATATABLE.equalsIgnoreCase(tableType) ? SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.LAST_MODIFY_TIME) : SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.LAST_MODIFY_TIME);
    for (MetadataEntityInfoItemPO syncInfoItem : syncInfoItems) {
      if (lastModifyTime.equalsIgnoreCase(syncInfoItem.getDataItemName()))
        continue; 
      MetadataEntityDataItemVO dataItem = dataItemMap.get(syncInfoItem.getDataItemName().toLowerCase());
      dataItem.setDataItemName(syncInfoItem.getDataItemName());
      ColumnParamDTO column = infoItemManager.createColumnParamDTO(dataItem, dataStructProvider.getDbType(), platCode, tokenId, syncInfoItem.getIsPrimaryKey(), syncInfoItem.getIsRequired());
      column.setIsPrimaryKey(DCConst.FALSE);
      if (DCConst.TableType.FILETABLE.equalsIgnoreCase(tableType))
        checkFileTable(column, dataStructProvider.getDbType()); 
      columns.add(column);
    } 
    table.setSchemaName(dataStructProvider.currentDataSource.getDbSchema());
    table.setTableName(historyTableName);
    table.setColumns(columns);
    try {
      String endTimeType;
      dataStructProvider.createTable(table);
      String pkFieldName = DCConst.TableType.DATATABLE.equalsIgnoreCase(tableType) ? SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID) : SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.ID);
      dataStructProvider.createIndex(historyTableName, pkFieldName, "dbid_" + WorkerId.getNextId(Long.class));
      dataStructProvider.addColumn(historyTableName, "lzdc_sys_version", 
          getDBColumnType(dataStructProvider.getDbType(), "Int", platCode, tokenId), DCConst.FALSE, DCConst.TRUE, 
          Integer.valueOf(4), Integer.valueOf(0));
      if ("mysql".equals(dataStructProvider.getDbType())) {
        endTimeType = "datetime";
      } else if ("sqlserver".equals(dataStructProvider.getDbType())) {
        endTimeType = "datetime2";
      } else {
        endTimeType = getDBColumnType(dataStructProvider.getDbType(), "DateTime", platCode, tokenId);
      } 
      dataStructProvider.addColumn(historyTableName, "lzdc_sys_end_time", endTimeType, DCConst.FALSE, DCConst.TRUE, 
          
          Integer.valueOf(3), Integer.valueOf(0));
      TableStructInfoDTO tableStructInfoDTO = getTableStructInfo(dataStructProvider, historyTableName);
      List<MetadataEntityInfoItemPO> needSetDesc = new ArrayList<>(syncInfoItems);
      if (DCConst.TableType.DATATABLE.equalsIgnoreCase(tableType))
        needSetDesc = (List<MetadataEntityInfoItemPO>)syncInfoItems.stream().filter(item -> !SysFieldHelper.sysFieldContains(item.getDataItemName())).collect(Collectors.toList()); 
      MetadataEntityInfoItemPO dbid = syncInfoItems.stream().filter(item -> pkFieldName.equalsIgnoreCase(item.getDataItemName())).findFirst().orElse(null);
      if (dbid != null) {
        dbid.setDataItemName(pkFieldName);
        needSetDesc.add(dbid);
      } 
      infoItemManager.setDBFieldDesc(tableStructInfoDTO, needSetDesc, dataStructProvider, historyTableName, platCode, tokenId);
    } catch (Exception e) {
      logErrorInfo(ExceptionUtil.getStackTraceMsg(e), "创建物理表【" + originTableName + "】的拉链表失败", platCode, tokenId);
    } 
  }
  
  private void checkFileTable(ColumnParamDTO column, String dbType) {
    if (SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.START_TIME).equalsIgnoreCase(column.getColumnName()) || 
      SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.LAST_MODIFY_TIME).equalsIgnoreCase(column.getColumnName()) || 
      SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.ACCESS_DATE).equalsIgnoreCase(column.getColumnName()))
      if ("mysql".equals(dbType)) {
        column.getColumnType().setMysqlDbType("datetime");
        column.setColumnLength(Integer.valueOf(3));
      } else if ("sqlserver".equals(dbType)) {
        column.getColumnType().setSqlDbType("datetime2");
        column.setColumnLength(Integer.valueOf(3));
      }  
  }
  
  private boolean checkAndCreateHistoryTable(DataStructProvider dataStructProvider, String originTableName, List<MetadataEntityInfoItemPO> syncInfoItems, Map<String, MetadataEntityDataItemVO> dataItemMap, String platCode, String tokenId, String tableType) {
    String historyTableName = getHistoryTableName(originTableName);
    Boolean existsTable = dataStructProvider.isExistsTable(historyTableName);
    if (existsTable == null || !existsTable.booleanValue()) {
      createHistoryTable(dataStructProvider, originTableName, syncInfoItems, dataItemMap, platCode, tokenId, tableType);
      return false;
    } 
    Boolean existVersionColumn = dataStructProvider.isExistsColumn(historyTableName, "kingbase".equals(dataStructProvider.getDbType()) ? "lzdc_sys_version".toLowerCase() : "lzdc_sys_version");
    if (existVersionColumn == null || !existVersionColumn.booleanValue()) {
      int tableDataCount = dataStructProvider.getTableDataCount(historyTableName);
      if (tableDataCount > 0) {
        logErrorInfo(null, "【" + historyTableName + "】表中缺少版本号字段，且已经存在数据，无法同步拉链表", platCode, tokenId);
      } else {
        dataStructProvider.dropTable(historyTableName);
        createHistoryTable(dataStructProvider, originTableName, syncInfoItems, dataItemMap, platCode, tokenId, tableType);
      } 
      return false;
    } 
    Boolean existEndTimeColumn = dataStructProvider.isExistsColumn(historyTableName, "kingbase".equals(dataStructProvider.getDbType()) ? "lzdc_sys_end_time".toLowerCase() : "lzdc_sys_end_time");
    if (existEndTimeColumn == null || !existEndTimeColumn.booleanValue())
      try {
        String endTimeType;
        if ("mysql".equals(dataStructProvider.getDbType())) {
          endTimeType = "datetime";
        } else if ("sqlserver".equals(dataStructProvider.getDbType())) {
          endTimeType = "datetime2";
        } else {
          endTimeType = getDBColumnType(dataStructProvider.getDbType(), "DateTime", platCode, tokenId);
        } 
        dataStructProvider.addColumn(historyTableName, "lzdc_sys_end_time", endTimeType, DCConst.FALSE, DCConst.TRUE, 
            
            Integer.valueOf(0), Integer.valueOf(0));
      } catch (Exception e) {
        logErrorInfo(ExceptionUtil.getStackTraceMsg(e), getColumnLogMsg(originTableName, "添加", "lzdc_sys_end_time"), platCode, tokenId);
      }  
    return true;
  }
  
  private boolean alterColumn(DataStructProvider dataStructProvider, String historyTableName, DBColumnDTO originTableColumn, MetadataEntityDataItemVO dataItem, String tokenId, String platCode) throws Exception {
    if (dataItem.getDataLength() == null)
      dataItem.setDataLength(Integer.valueOf(1)); 
    if (dataItem.getDataScale() == null)
      dataItem.setDataScale(Integer.valueOf(0)); 
    return dataStructProvider.alterColumn(historyTableName, dataItem
        .getDataLength().intValue(), dataItem
        .getDataScale().intValue(), (
        (IInfoItemSync)ServiceUtil.getInfoItemSyncManager(platCode, IInfoItemSync.class)).getDBColumnType(dataStructProvider.getDbType(), dataItem.getDataType(), platCode, tokenId), DCConst.TRUE
        .equals(originTableColumn.getIsNullAble()), originTableColumn
        .getColumnName(), DCConst.TRUE
        .equals(originTableColumn.getIsPrimaryKey()));
  }
  
  public void batchBackup(DataStructProvider dataStructProvider, String originTableName, List<Map<String, Object>> dataList, String platCode, String tokenId, String tableType, List<String> masterPrimaryKeyList, Long tableId, Boolean isCompareMap, Integer isRes) {
    if (!getEnableHistory(platCode, tableType).booleanValue() || dataList == null || dataList.isEmpty())
      return; 
    Boolean isExistHistoryTable = (Boolean)checkHistoryTableName(dataStructProvider, originTableName, platCode, tableType).getObj();
    if ((isExistHistoryTable == null || !isExistHistoryTable.booleanValue()) && DCConst.TableType.FILETABLE.equals(tableType)) {
      saveHistoryTableForFileTable(dataStructProvider, tableId, originTableName, platCode, tokenId);
    } else if ((isExistHistoryTable == null || !isExistHistoryTable.booleanValue()) && DCConst.TableType.DATATABLE.equals(tableType)) {
      saveHistoryTable(dataStructProvider, tableId, originTableName, platCode, tokenId, isRes);
    } 
    String historyTableName = getHistoryTableName(originTableName);
    List<String> backupColumns = getBackupColumnsLowerCase(dataStructProvider, originTableName, historyTableName);
    if (backupColumns.isEmpty() || !isMeetBackupCondition(backupColumns))
      return; 
    if (DCConst.TableType.DATATABLE.equals(tableType)) {
      masterPrimaryKeyList = new ArrayList<>();
      masterPrimaryKeyList.add(SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID));
    } else if (DCConst.TableType.FILETABLE.equals(tableType)) {
      masterPrimaryKeyList = new ArrayList<>();
      masterPrimaryKeyList.add(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.ID));
    } 
    for (Map<String, Object> data : dataList) {
      try {
        backup(dataStructProvider, historyTableName, data, backupColumns, masterPrimaryKeyList, tableType, isCompareMap);
      } catch (CustomException e) {
        logErrorInfo(null, "备份【" + originTableName + "】数据失败，原因：" + e.getMessage() + "，数据为：" + JSON.toJSONString(data), platCode, tokenId);
      } catch (Exception e) {
        logErrorInfo(ExceptionUtil.getStackTraceMsg(e), "备份【" + originTableName + "】数据失败，数据为：" + JSON.toJSONString(data), platCode, tokenId);
      } 
    } 
  }
  
  public void backup(DataStructProvider dataStructProvider, String originTableName, Map<String, Object> data, String platCode, String tokenId, String tableType, List<String> masterPrimaryKeyList) {
    if (!getEnableHistory(platCode, tableType).booleanValue())
      return; 
    String historyTableName = getHistoryTableName(originTableName);
    List<String> backupColumns = getBackupColumnsLowerCase(dataStructProvider, originTableName, historyTableName);
    if (backupColumns.isEmpty() || !isMeetBackupCondition(backupColumns))
      return; 
    try {
      if (DCConst.TableType.DATATABLE.equals(tableType)) {
        masterPrimaryKeyList = new ArrayList<>();
        masterPrimaryKeyList.add(SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID));
      } 
      backup(dataStructProvider, historyTableName, data, backupColumns, masterPrimaryKeyList, tableType, null);
    } catch (CustomException e) {
      logErrorInfo(null, "备份【" + originTableName + "】数据失败，原因：" + e.getMessage() + "，数据为：" + JSON.toJSONString(data), platCode, tokenId);
    } catch (Exception e) {
      logErrorInfo(ExceptionUtil.getStackTraceMsg(e), "备份【" + originTableName + "】数据失败，数据为：" + JSON.toJSONString(data), platCode, tokenId);
    } 
  }
  
  public void batchDeleteData(DataStructProvider dataStructProvider, String originTableName, List<Map<String, Object>> dataList, String platCode, String tokenId, String tableType, List<String> masterPrimaryKeyList) {
    if (!getEnableHistory(platCode, tableType).booleanValue())
      return; 
    String historyTableName = getHistoryTableName(originTableName);
    List<String> historyColumns = (List<String>)dataStructProvider.getEntityDbTableStruct(historyTableName, null).stream().map(item -> item.getColumnName().toLowerCase()).collect(Collectors.toList());
    if (!isMeetBackupCondition(historyColumns))
      return; 
    if (DCConst.TableType.DATATABLE.equals(tableType)) {
      masterPrimaryKeyList = new ArrayList<>();
      masterPrimaryKeyList.add(SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID));
    } else if (DCConst.TableType.FILETABLE.equals(tableType)) {
      masterPrimaryKeyList = new ArrayList<>();
      masterPrimaryKeyList.add(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.ID));
    } 
    for (Map<String, Object> data : dataList) {
      try {
        deleteData(dataStructProvider, historyTableName, data, masterPrimaryKeyList, tableType);
      } catch (CustomException e) {
        logErrorInfo(null, "删除【" + originTableName + "】拉链表数据失败，原因：" + e.getMessage() + "，数据为：" + JSON.toJSONString(data), platCode, tokenId);
      } catch (Exception e) {
        logErrorInfo(ExceptionUtil.getStackTraceMsg(e), "删除【" + originTableName + "】拉链表数据失败，数据为：" + JSON.toJSONString(data), platCode, tokenId);
      } 
    } 
  }
  
  public void deleteData(DataStructProvider dataStructProvider, String originTableName, Map<String, Object> data, String platCode, String tokenId, String tableType, List<String> masterPrimaryKeyList) {
    if (!getEnableHistory(platCode, tableType).booleanValue())
      return; 
    String historyTableName = getHistoryTableName(originTableName);
    List<String> historyColumns = (List<String>)dataStructProvider.getEntityDbTableStruct(historyTableName, null).stream().map(item -> item.getColumnName().toLowerCase()).collect(Collectors.toList());
    if (!isMeetBackupCondition(historyColumns))
      return; 
    try {
      if (DCConst.TableType.DATATABLE.equals(tableType)) {
        masterPrimaryKeyList = new ArrayList<>();
        masterPrimaryKeyList.add(SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID));
      } 
      deleteData(dataStructProvider, historyTableName, data, masterPrimaryKeyList, tableType);
    } catch (CustomException e) {
      logErrorInfo(null, "删除【" + originTableName + "】拉链表数据失败，原因：" + e.getMessage() + "，数据为：" + JSON.toJSONString(data), platCode, tokenId);
    } catch (Exception e) {
      logErrorInfo(ExceptionUtil.getStackTraceMsg(e), "删除【" + originTableName + "】拉链表数据失败，数据为：" + JSON.toJSONString(data), platCode, tokenId);
    } 
  }
  
  public void saveHistoryTableForFileTable(DataStructProvider dataStructProvider, Long fileTableId, String originTableName, String platCode, String tokenId) {
    saveHistoryTableForFileTable(dataStructProvider, fileTableId, originTableName, platCode, tokenId, DCConst.DataResourceCategory.DATA);
  }
  
  public void saveHistoryTableForFileTable(DataStructProvider dataStructProvider, Long fileTableId, String originTableName, String platCode, String tokenId, Integer dataCategory) {
    if (!getEnableHistory(platCode, DCConst.TableType.FILETABLE).booleanValue())
      return; 
    Map<String, DBColumnDTO> originTableColumnMap = (Map<String, DBColumnDTO>)dataStructProvider.getEntityDbTableStruct(originTableName, null).stream().filter(item -> !SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.LAST_MODIFY_TIME).equalsIgnoreCase(item.getColumnName())).collect(Collectors.toMap(item -> item.getColumnName().toLowerCase(), Function.identity(), (k1, k2) -> k1));
    List<ResAndDataFileFieldDTO> fileInfoFieldList = FileInfoManager.selectFileInfoFieldsByFileTableId(fileTableId, platCode, dataCategory);
    List<MetadataEntityInfoItemPO> syncInfoItems = new ArrayList<>();
    Map<String, MetadataEntityDataItemVO> dataItemMap = new HashMap<>();
    getFieldAndSysFieldsForFileTable(fileInfoFieldList, syncInfoItems, dataItemMap);
    boolean isPass = checkAndCreateHistoryTable(dataStructProvider, originTableName, syncInfoItems, dataItemMap, platCode, tokenId, DCConst.TableType.FILETABLE);
    if (!isPass)
      return; 
    String historyTableName = getHistoryTableName(originTableName);
    Map<String, DBColumnDTO> historyTableColumnMap = (Map<String, DBColumnDTO>)dataStructProvider.getEntityDbTableStruct(historyTableName, null).stream().collect(Collectors.toMap(item -> item.getColumnName().toLowerCase(), Function.identity(), (k1, k2) -> k1));
    for (DBColumnDTO originTableColumn : originTableColumnMap.values()) {
      String lowerCaseColumnName = originTableColumn.getColumnName().toLowerCase();
      MetadataEntityDataItemVO dataItem = dataItemMap.get(lowerCaseColumnName);
      if (dataItem == null)
        continue; 
      DBColumnDTO historyTableColumn = historyTableColumnMap.get(lowerCaseColumnName);
      try {
        if (historyTableColumn != null) {
          if (!Objects.equals(originTableColumn.getDataType(), historyTableColumn.getDataType()) || 
            !Objects.equals(originTableColumn.getDataLength(), historyTableColumn.getDataLength()) || 
            !Objects.equals(originTableColumn.getScale(), historyTableColumn.getScale()) || 
            !Objects.equals(originTableColumn.getIsNullAble(), historyTableColumn.getIsNullAble())) {
            boolean bool = alterColumn(dataStructProvider, historyTableName, originTableColumn, dataItem, tokenId, platCode);
            if (!bool)
              logErrorInfo(null, getColumnLogMsg(originTableName, "修改", originTableColumn.getColumnName()), platCode, tokenId); 
          } 
          continue;
        } 
        boolean success = dataStructProvider.createColumn(historyTableName, originTableColumn.getColumnName(), (
            (IInfoItemSync)ServiceUtil.getInfoItemSyncManager(platCode, IInfoItemSync.class)).getDBColumnType(dataStructProvider.getDbType(), dataItem.getDataType(), platCode, tokenId), dataItem
            .getDataLength(), dataItem.getDataScale(), DCConst.FALSE.equals(originTableColumn.getIsNullAble())).booleanValue();
        if (!success)
          logErrorInfo(null, getColumnLogMsg(originTableName, "添加", originTableColumn.getColumnName()), platCode, tokenId); 
      } catch (Exception e) {
        logErrorInfo(ExceptionUtil.getStackTraceMsg(e), getColumnLogMsg(originTableName, "处理", originTableColumn.getColumnName()), platCode, tokenId);
      } 
    } 
    DBColumnDTO dbColumnDTO = historyTableColumnMap.get("lzdc_sys_end_time");
    checkEndTimeColumn(dataStructProvider, historyTableName, dbColumnDTO);
    TableStructInfoDTO tableStructInfoDTO = getTableStructInfo(dataStructProvider, historyTableName);
    List<MetadataEntityInfoItemPO> needSetDesc = (List<MetadataEntityInfoItemPO>)syncInfoItems.stream().filter(item -> !SysFieldHelper.sysFieldContains(item.getDataItemName())).collect(Collectors.toList());
    ((IInfoItem)ServiceUtil.getInfoItemManager(platCode, IInfoItem.class)).setDBFieldDesc(tableStructInfoDTO, needSetDesc, dataStructProvider, historyTableName, platCode, tokenId);
  }
  
  private void deleteData(DataStructProvider dataStructProvider, String historyTableName, Map<String, Object> data, List<String> masterPrimaryKeyList, String tableType) throws Exception {
    QueryCondition pkQueryCondition = getPkQueryCondition(dataStructProvider.getDbType(), data, masterPrimaryKeyList);
    if (pkQueryCondition.getParam().isEmpty())
      return; 
    int nextVersion = getNextVersion(dataStructProvider, historyTableName, pkQueryCondition);
    if (nextVersion > 1)
      updateEndTime(dataStructProvider, historyTableName, pkQueryCondition, nextVersion - 1, data, tableType); 
    StringBuilder deleteSql = new StringBuilder();
    if (DCConst.TableType.DATATABLE.equalsIgnoreCase(tableType)) {
      deleteSql.append(" update ").append(TypeConverter.getTableNameSqlStr(historyTableName, dataStructProvider.getDbType())).append(" set ")
        .append(SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.IS_DELETE)).append(" = 1 ")
        .append(" where ").append(pkQueryCondition.getWhereSql());
    } else if (DCConst.TableType.FILETABLE.equalsIgnoreCase(tableType)) {
      deleteSql.append(" update ").append(TypeConverter.getTableNameSqlStr(historyTableName, dataStructProvider.getDbType())).append(" set ")
        .append(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.IS_DELETE)).append(" = 1 ")
        .append(" where ").append(pkQueryCondition.getWhereSql());
    } 
    Integer deleteResult = dataStructProvider.execCustomSql(deleteSql.toString(), pkQueryCondition.getParam());
    if (deleteResult == null || deleteResult.intValue() < 1)
      throw new CustomException("删除语句执行失败，SQL：【" + deleteSql + "】,参数：【" + JSON.toJSONString(pkQueryCondition.getParam()) + "】"); 
  }
  
  private void backup(DataStructProvider dataStructProvider, String historyTableName, Map<String, Object> data, List<String> backupColumns, List<String> masterPrimaryKeyList, String tableType, Boolean isCompareMap) throws Exception {
    boolean changed;
    if (data == null || data.isEmpty())
      return; 
    if (isCompareMap != null && isCompareMap.booleanValue()) {
      changed = dataIsChangedByCompareMap(dataStructProvider, historyTableName, data, tableType);
    } else {
      changed = dataIsChanged(dataStructProvider, historyTableName, data);
    } 
    if (!changed)
      return; 
    QueryCondition pkQueryCondition = getPkQueryCondition(dataStructProvider.getDbType(), data, masterPrimaryKeyList);
    if (pkQueryCondition.getParam().isEmpty())
      throw new CustomException("主键列值为空"); 
    int nextVersion = getNextVersion(dataStructProvider, historyTableName, pkQueryCondition);
    if (nextVersion > 1)
      updateEndTime(dataStructProvider, historyTableName, pkQueryCondition, nextVersion - 1, data, tableType); 
    Map<String, Object> insertParam = prepareInsertParam(data, backupColumns, nextVersion, tableType);
    StringBuilder insertColumnNameSql = new StringBuilder();
    StringBuilder insertColumnValueSql = new StringBuilder();
    for (String key : insertParam.keySet()) {
      if (insertColumnNameSql.length() == 0) {
        insertColumnNameSql.append(TypeConverter.getColumnSqlStr(key, dataStructProvider.getDbType()));
      } else {
        insertColumnNameSql.append(",").append(TypeConverter.getColumnSqlStr(key, dataStructProvider.getDbType()));
      } 
      if (insertColumnValueSql.length() == 0) {
        insertColumnValueSql.append("#{").append(key).append("}");
        continue;
      } 
      insertColumnValueSql.append(",").append("#{").append(key).append("}");
    } 
    StringBuilder insertSql = new StringBuilder();
    insertSql.append("insert into ").append(TypeConverter.getTableNameSqlStr(historyTableName, dataStructProvider.getDbType())).append("(");
    if (insertColumnNameSql.length() > 0)
      insertSql.append(insertColumnNameSql).append(") "); 
    if (insertColumnValueSql.length() > 0)
      insertSql.append("values (").append(insertColumnValueSql).append(")"); 
    Integer backupResult = dataStructProvider.execCustomSql(insertSql.toString(), insertParam);
    if (backupResult == null || backupResult.intValue() < 1)
      throw new CustomException("备份语句执行失败，SQL：【" + insertSql + "】,参数：【" + JSON.toJSONString(insertParam) + "】"); 
  }
  
  private boolean dataIsChanged(DataStructProvider dataStructProvider, String historyTableName, Map<String, Object> data) {
    Map<String, Object> customFields = (Map<String, Object>)data.entrySet().stream().filter(item -> !SysFieldHelper.sysFieldContains((String)item.getKey())).collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue, (k1, k2) -> k1));
    StringBuilder whereSql = new StringBuilder();
    whereSql.append(TypeConverter.getColumnSqlStr(SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.IS_DELETE), dataStructProvider.getDbType())).append("=0 ");
    Map<String, Object> param = new HashMap<>(customFields.size());
    for (Map.Entry<String, Object> entry : customFields.entrySet()) {
      whereSql.append(" and ").append(TypeConverter.getColumnSqlStr(entry.getKey(), dataStructProvider.getDbType()))
        .append("=#{").append(entry.getKey()).append("}");
      param.put(entry.getKey(), entry.getValue());
    } 
    String selectCountSql = "select count(1)  from " + TypeConverter.getTableNameSqlStr(historyTableName, dataStructProvider.getDbType()) + " where " + whereSql.toString();
    Integer count = dataStructProvider.execCountSql(selectCountSql, param);
    return (count == null || count.intValue() < 1);
  }
  
  private boolean dataIsChangedByCompareMap(DataStructProvider dataStructProvider, String historyTableName, Map<String, Object> data, String tableType) {
    StringBuilder whereSql = new StringBuilder();
    whereSql.append(TypeConverter.getColumnSqlStr(SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.IS_DELETE), dataStructProvider.getDbType())).append("=0 ");
    whereSql.append(" and ");
    if (dataStructProvider.currentDataSource.getType().equalsIgnoreCase("postgresql")) {
      if (DCConst.TableType.DATATABLE.equals(tableType)) {
        whereSql.append(TypeConverter.getColumnSqlStr(SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID).toLowerCase(), dataStructProvider.getDbType())).append("='").append(data.get(SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID).toLowerCase())).append("'");
      } else {
        whereSql.append(TypeConverter.getColumnSqlStr(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.ID).toLowerCase(), dataStructProvider.getDbType())).append("='").append(data.get(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.ID).toLowerCase())).append("'");
      } 
    } else if (DCConst.TableType.DATATABLE.equals(tableType)) {
      whereSql.append(TypeConverter.getColumnSqlStr(SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID), dataStructProvider.getDbType())).append("='").append(data.get(SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID))).append("'");
    } else {
      whereSql.append(TypeConverter.getColumnSqlStr(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.ID), dataStructProvider.getDbType())).append("='").append(data.get(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.ID))).append("'");
    } 
    String selectCountSql = "select *  from " + TypeConverter.getTableNameSqlStr(historyTableName, dataStructProvider.getDbType()) + " where " + whereSql.toString();
    List<Map<String, Object>> oldDataList = dataStructProvider.execCommonQuery(selectCountSql);
    if (!CollectionUtils.isEmpty(oldDataList)) {
      List<Map<String, Object>> sortedList = (List<Map<String, Object>>)oldDataList.stream().sorted(Comparator.comparingInt(s -> ((Integer)s.getOrDefault("lzdc_sys_version", Integer.valueOf(0))).intValue())).collect(Collectors.toList());
      Map<String, Object> maxVersionData = sortedList.isEmpty() ? null : sortedList.get(sortedList.size() - 1);
      return checkDataIsChange(maxVersionData, data, tableType);
    } 
    return true;
  }
  
  private static boolean checkDataIsChange(Map<String, Object> beforeUpdateDataMap, Map<String, Object> afterUpdateDataMap, String tableType) {
    Set<String> handledKeys = new HashSet<>();
    for (Map.Entry<String, Object> entry : afterUpdateDataMap.entrySet()) {
      handledKeys.add(entry.getKey());
      if (HIS_TABLE_FIELD.contains(entry.getKey()))
        continue; 
      if (DCConst.TableType.FILETABLE.equals(tableType) && SysFieldHelper.fileSysFieldContains(entry.getKey()))
        continue; 
      if (DCConst.TableType.DATATABLE.equals(tableType) && SysFieldHelper.sysFieldContains(entry.getKey()))
        continue; 
      if (!Objects.equals(entry.getValue(), beforeUpdateDataMap.get(entry.getKey())))
        return true; 
    } 
    for (Map.Entry<String, Object> entry : beforeUpdateDataMap.entrySet()) {
      if (HIS_TABLE_FIELD.contains(entry.getKey()))
        continue; 
      if (handledKeys.contains(entry.getKey()))
        continue; 
      if (DCConst.TableType.FILETABLE.equals(tableType) && SysFieldHelper.fileSysFieldContains(entry.getKey()))
        continue; 
      if (DCConst.TableType.DATATABLE.equals(tableType) && SysFieldHelper.sysFieldContains(entry.getKey()))
        continue; 
      if (!Objects.equals(entry.getValue(), afterUpdateDataMap.get(entry.getKey())))
        return true; 
    } 
    return false;
  }
  
  private Map<String, Object> prepareInsertParam(Map<String, Object> data, List<String> backupColumns, int nextVersion, String tableType) throws ParseException {
    String isDelFieldName = DCConst.TableType.FILETABLE.equalsIgnoreCase(tableType) ? SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.IS_DELETE) : SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.IS_DELETE);
    String lastModfyFieldName = DCConst.TableType.FILETABLE.equalsIgnoreCase(tableType) ? SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.LAST_MODIFY_TIME) : SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.LAST_MODIFY_TIME);
    String startTimeFieldName = DCConst.TableType.FILETABLE.equalsIgnoreCase(tableType) ? SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.START_TIME) : SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.START_TIME);
    Map<String, Object> insertParam = (Map<String, Object>)data.entrySet().stream().filter(item -> (backupColumns.contains(((String)item.getKey()).toLowerCase()) || SysFieldHelper.sysFieldContains((String)item.getKey()))).filter(item -> (!"lzdc_sys_version".equals(((String)item.getKey()).toLowerCase()) && !isDelFieldName.equalsIgnoreCase(((String)item.getKey()).toLowerCase()) && !lastModfyFieldName.equalsIgnoreCase(((String)item.getKey()).toLowerCase()) && !startTimeFieldName.equalsIgnoreCase(((String)item.getKey()).toLowerCase()) && item.getValue() != null)).collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
    if (DCConst.TableType.FILETABLE.equalsIgnoreCase(tableType))
      insertParam = (Map<String, Object>)data.entrySet().stream().filter(item -> backupColumns.contains(((String)item.getKey()).toLowerCase())).filter(item -> (!"lzdc_sys_version".equals(((String)item.getKey()).toLowerCase()) && !isDelFieldName.equalsIgnoreCase(((String)item.getKey()).toLowerCase()) && !lastModfyFieldName.equalsIgnoreCase(((String)item.getKey()).toLowerCase()) && !startTimeFieldName.equalsIgnoreCase(((String)item.getKey()).toLowerCase()) && item.getValue() != null)).collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue)); 
    if (insertParam.isEmpty())
      throw new CustomException("字段值全部为空"); 
    insertParam.put(isDelFieldName, DCConst.FALSE);
    insertParam.put("lzdc_sys_version", Integer.valueOf(nextVersion));
    DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    if (nextVersion > 1) {
      Object startTime = data.entrySet().stream().filter(item -> lastModfyFieldName.equalsIgnoreCase((String)item.getKey())).map(Map.Entry::getValue).findFirst().orElse(null);
      insertParam.put(startTimeFieldName, startTime);
    } else {
      Object updateTime = data.entrySet().stream().filter(item -> startTimeFieldName.equalsIgnoreCase((String)item.getKey())).map(Map.Entry::getValue).findFirst().orElse(null);
      insertParam.put(startTimeFieldName, updateTime);
    } 
    insertParam.put("lzdc_sys_end_time", dateFormat.parse("9999-12-31 23:59:59"));
    return new HashMap<>(insertParam);
  }
  
  private Map<String, Object> prepareInsertParamForFileTable(Map<String, Object> data, List<String> backupColumns, int nextVersion) throws ParseException {
    Map<String, Object> insertParam = (Map<String, Object>)data.entrySet().stream().filter(item -> (backupColumns.contains(((String)item.getKey()).toLowerCase()) || SysFieldHelper.sysFieldContains((String)item.getKey()))).filter(item -> (!"lzdc_sys_version".equals(((String)item.getKey()).toLowerCase()) && !SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.IS_DELETE).equalsIgnoreCase(((String)item.getKey()).toLowerCase()) && !SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.LAST_MODIFY_TIME).equalsIgnoreCase(((String)item.getKey()).toLowerCase()) && !SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.START_TIME).equalsIgnoreCase(((String)item.getKey()).toLowerCase()))).collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
    if (insertParam.isEmpty())
      throw new CustomException("字段值全部为空"); 
    insertParam.put(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.IS_DELETE), DCConst.FALSE);
    insertParam.put("lzdc_sys_version", Integer.valueOf(nextVersion));
    DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    if (nextVersion > 1) {
      Object startTime = data.entrySet().stream().filter(item -> SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.LAST_MODIFY_TIME).equalsIgnoreCase((String)item.getKey())).map(Map.Entry::getValue).findFirst().orElse(null);
      insertParam.put(SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.START_TIME), startTime);
    } else {
      Object updateTime = data.entrySet().stream().filter(item -> SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.START_TIME).equalsIgnoreCase((String)item.getKey())).map(Map.Entry::getValue).findFirst().orElse(null);
      insertParam.put(SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.START_TIME), updateTime);
    } 
    insertParam.put("lzdc_sys_end_time", dateFormat.parse("9999-12-31 23:59:59"));
    return new HashMap<>(insertParam);
  }
  
  private void updateEndTime(DataStructProvider dataStructProvider, String historyTableName, QueryCondition pkQueryCondition, int version, Map<String, Object> data, String tableType) throws Exception {
    String updateWhereSql = pkQueryCondition.getWhereSql() + " and " + "lzdc_sys_version" + " = " + version;
    String sql = "update " + TypeConverter.getTableNameSqlStr(historyTableName, dataStructProvider.getDbType()) + " set " + "lzdc_sys_end_time" + " = #{" + "lzdc_sys_end_time" + "} where " + updateWhereSql;
    Map<String, Object> updateParam = new HashMap<>(pkQueryCondition.getParam());
    if (DCConst.TableType.DATATABLE.equalsIgnoreCase(tableType)) {
      Object updateTime = data.entrySet().stream().filter(item -> SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.LAST_MODIFY_TIME).equalsIgnoreCase((String)item.getKey())).map(Map.Entry::getValue).findFirst().orElse(null);
      updateParam.put("lzdc_sys_end_time", updateTime);
    } else if (DCConst.TableType.FILETABLE.equalsIgnoreCase(tableType)) {
      Object updateTime = data.entrySet().stream().filter(item -> SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.LAST_MODIFY_TIME).equalsIgnoreCase((String)item.getKey())).map(Map.Entry::getValue).findFirst().orElse(null);
      updateParam.put("lzdc_sys_end_time", updateTime);
    } 
    dataStructProvider.execCustomSql(sql, updateParam);
  }
  
  private List<String> getBackupColumnsLowerCase(DataStructProvider dataStructProvider, String originTableName, String historyTableName) {
    Map<String, DBColumnDTO> originTableColumnMap = (Map<String, DBColumnDTO>)dataStructProvider.getEntityDbTableStruct(originTableName, null).stream().collect(Collectors.toMap(item -> item.getColumnName().toLowerCase(), Function.identity(), (k1, k2) -> k1));
    return (List<String>)dataStructProvider.getEntityDbTableStruct(historyTableName, null).stream().filter(item -> {
          if (SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID).equals(item.getColumnName().toLowerCase()) || "lzdc_sys_version".equals(item.getColumnName().toLowerCase()) || "lzdc_sys_end_time".equals(item.getColumnName().toLowerCase()))
            return true; 
          DBColumnDTO originColumn = (DBColumnDTO)originTableColumnMap.get(item.getColumnName().toLowerCase());
          return (originColumn != null && Objects.equals(originColumn.getDataType(), item.getDataType()));
        }).map(item -> item.getColumnName().toLowerCase()).collect(Collectors.toList());
  }
  
  private boolean isMeetBackupCondition(List<String> lowerCaseColumns) {
    return (lowerCaseColumns.contains("lzdc_sys_version".toLowerCase()) && lowerCaseColumns.contains("lzdc_sys_end_time".toLowerCase()) && lowerCaseColumns
      .contains(SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID).toLowerCase()));
  }
  
  private QueryCondition getPkQueryCondition(String dbType, Map<String, Object> data, List<String> masterPrimaryKeyList) {
    Map<String, Object> param = new HashMap<>();
    for (Map.Entry<String, Object> entry : data.entrySet()) {
      for (String pkField : masterPrimaryKeyList) {
        if (pkField.equalsIgnoreCase(entry.getKey()) && entry.getValue() != null)
          param.put(pkField, entry.getValue()); 
      } 
    } 
    QueryCondition queryCondition = new QueryCondition(null);
    StringBuilder whereSql = (new StringBuilder()).append(TypeConverter.getColumnSqlStr(SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.IS_DELETE), dbType) + "=0 ");
    for (String pkField : masterPrimaryKeyList)
      whereSql.append(" and " + TypeConverter.getColumnSqlStr(pkField, dbType) + "=#{" + pkField + "} "); 
    queryCondition.setWhereSql(whereSql.toString());
    queryCondition.setParam(param);
    return queryCondition;
  }
  
  private int getNextVersion(DataStructProvider dataStructProvider, String historyTableName, QueryCondition queryCondition) {
    String selectCountSql = "select count(1)  from " + TypeConverter.getTableNameSqlStr(historyTableName, dataStructProvider.getDbType()) + " where " + queryCondition.getWhereSql();
    Integer count = dataStructProvider.execCountSql(selectCountSql, queryCondition.getParam());
    return (count == null) ? 1 : (count.intValue() + 1);
  }
  
  private String getHistoryTableName(String originTableName) {
    return originTableName + "_history";
  }
  
  private String getColumnLogMsg(String tableName, String option, String columnName) {
    String format = "物理表【%s】的拉链表%s字段【%s】失败";
    return String.format(format, new Object[] { tableName, option, columnName });
  }
  
  private void checkEndTimeColumn(DataStructProvider dataStructProvider, String historyTableName, DBColumnDTO dbColumnDTO) {
    if (dbColumnDTO != null)
      try {
        if ("mysql".equals(dataStructProvider.getDbType()) && !"datetime(3)".equalsIgnoreCase(dbColumnDTO.getFullColumnType())) {
          dataStructProvider.alterColumn(historyTableName, 3, 0, "datetime", true, "lzdc_sys_end_time", false);
        } else if ("sqlserver".equals(dataStructProvider.getDbType()) && !"datetime2".equalsIgnoreCase(dbColumnDTO.getColumnName())) {
          dataStructProvider.alterColumn(historyTableName, 3, 0, "datetime2", true, "lzdc_sys_end_time", false);
        } 
      } catch (Exception exception) {} 
  }
  
  private void logErrorInfo(String stackInfo, String msg, String platCode, String tokenId) {
    CommonLogDTO logDTO = new CommonLogDTO();
    logDTO.setPlatCode(platCode);
    logDTO.setTokenId(tokenId);
    logDTO.setStackInformation(stackInfo);
    logDTO.setLoggerInformation(msg);
    logDTO.setLogLevel(FrameConst.Log.LOG_LEVEL_ERROR);
    logDTO.setLogType(0);
    logDTO.setModule_code(HeaderInterceptor.getModuleCode());
    RabbitMQUtil.publish("DC.Standard.log", logDTO);
  }
  
  public String getDBColumnType(String dbType, String dataType, String platCode, String tokenId) {
    MetadataTypeVO metadataTypeVO = (MetadataTypeVO)this.dataStandardClient.getMetadataTypeByDataTypeName(dataType, platCode, tokenId).getObj();
    String dbColumnType = "";
    if (dbType.equalsIgnoreCase("mysql")) {
      dbColumnType = metadataTypeVO.getMysqlDbType();
    } else if (dbType.equalsIgnoreCase("sqlserver")) {
      dbColumnType = metadataTypeVO.getSqlDbType();
    } else if (dbType.equalsIgnoreCase("oracle")) {
      dbColumnType = metadataTypeVO.getOracleDbType();
    } else if (dbType.equalsIgnoreCase("postgresql")) {
      dbColumnType = metadataTypeVO.getPgsqlDbType();
    } else if (dbType.equalsIgnoreCase("dm")) {
      dbColumnType = metadataTypeVO.getDm8DbType();
    } else if (dbType.equalsIgnoreCase("kingbase")) {
      dbColumnType = metadataTypeVO.getKingBaseDbType();
    } 
    return dbColumnType;
  }
  
  private TableStructInfoDTO getTableStructInfo(DataStructProvider dataStructProvider, String tableName) {
    TableStructInfoDTO tableStructInfoDTO = new TableStructInfoDTO();
    Map<String, String> tableStruct = new HashMap<>();
    Map<String, Integer> tableIsNull = new HashMap<>();
    List<String> fieldList = new ArrayList<>();
    List<String> pkFields = new ArrayList<>();
    List<DBColumnDTO> tableStructDt = dataStructProvider.getEntityDbTableStruct(tableName, null);
    if (tableStructDt != null && tableStructDt.size() > 0)
      tableStructDt.forEach(item -> {
            tableStruct.put(item.getColumnName().toLowerCase(), item.getDataType());
            tableIsNull.put(item.getColumnName().toLowerCase(), item.getIsNullAble());
            fieldList.add(item.getColumnName());
            if (Objects.equals(item.getIsPrimaryKey(), DCConst.TRUE))
              pkFields.add(item.getColumnName()); 
          }); 
    tableStructInfoDTO.setFieldList(fieldList);
    tableStructInfoDTO.setPkFields(pkFields);
    tableStructInfoDTO.setTableIsNull(tableIsNull);
    tableStructInfoDTO.setTableStruct(tableStruct);
    return tableStructInfoDTO;
  }
  
  private Boolean createSysField(List<MetadataEntityInfoItemPO> syncInfoItems, Map<String, MetadataEntityDataItemVO> dataItemMap, String fieldName, String dataTypeName) {
    int defaultLength = 50;
    int longLength = 20;
    int intLength = 4;
    MetadataEntityInfoItemPO infoItem = null;
    boolean isExist = false;
    if (infoItem == null) {
      int dataLength = 1;
      if (Objects.equals("String", dataTypeName)) {
        dataLength = 50;
      } else if (Objects.equals("Long", dataTypeName)) {
        dataLength = 20;
      } else if (Objects.equals("Int", dataTypeName)) {
        dataLength = 4;
      } 
      createInfoItemAndDataItem(syncInfoItems, dataItemMap, fieldName, DCConst.FALSE.intValue(), dataTypeName, dataLength, 0);
    } else {
      isExist = true;
    } 
    return Boolean.valueOf(isExist);
  }
  
  public void createInfoItemAndDataItem(List<MetadataEntityInfoItemPO> syncInfoItems, Map<String, MetadataEntityDataItemVO> dataItemMap, String itemName, int isPrimaryKey, String dbDataTypeName, int length, int scale) {
    MetadataEntityDataItemVO tempDataItem = new MetadataEntityDataItemVO();
    tempDataItem.setDataItemId((Long)WorkerId.getNextId(Long.class));
    tempDataItem.setDataType(dbDataTypeName);
    tempDataItem.setDataLength(Integer.valueOf(length));
    tempDataItem.setDataScale(Integer.valueOf(scale));
    tempDataItem.setDataScale(Integer.valueOf(scale));
    tempDataItem.setDataItemName(itemName);
    tempDataItem.setDataRule("");
    tempDataItem.setDataRuleDescription("");
    tempDataItem.setDataItemType("0");
    tempDataItem.setIsDelete(DCConst.FALSE);
    tempDataItem.setCreateTime(new Date());
    MetadataEntityInfoItemPO tempInfoItem = new MetadataEntityInfoItemPO();
    tempInfoItem.setInfoItemId((Long)WorkerId.getNextId(Long.class));
    tempInfoItem.setDataItemName(itemName);
    tempInfoItem.setInfoItemType(Integer.valueOf(0));
    tempInfoItem.setIsPrimaryKey(Integer.valueOf(isPrimaryKey));
    tempInfoItem.setInfoItemOrder(Integer.valueOf(1));
    tempInfoItem.setDataItemId(tempDataItem.getDataItemId());
    tempInfoItem.setInfoItemName(itemName);
    tempInfoItem.setCreateTime(new Date());
    tempInfoItem.setIsDelete(DCConst.FALSE);
    tempInfoItem.setIsRequired(Integer.valueOf(DCConst.TRUE.equals(Integer.valueOf(isPrimaryKey)) ? 1 : 0));
    tempInfoItem.setDescription("");
    tempInfoItem.setInfoItemStatus(DCConst.InfoItemStatus.VERIFY);
    tempInfoItem.setIsReference(DCConst.FALSE);
    syncInfoItems.add(tempInfoItem);
    dataItemMap.put(itemName.toLowerCase(), tempDataItem);
  }
  
  private void getFieldAndSysFieldsForFileTable(List<ResAndDataFileFieldDTO> fileInfoFieldList, List<MetadataEntityInfoItemPO> syncInfoItems, Map<String, MetadataEntityDataItemVO> dataItemMap) {
    for (FileInfoFieldPO fieldPO : fileInfoFieldList) {
      if (DCConst.FALSE.equals(fieldPO.getFieldStatus()))
        continue; 
      MetadataEntityDataItemVO dataItemVO = new MetadataEntityDataItemVO();
      dataItemVO.setDataItemName(fieldPO.getFieldName());
      dataItemVO.setDataLength(fieldPO.getFieldLength());
      dataItemVO.setDataScale(fieldPO.getFieldScale());
      dataItemVO.setDataType(fieldPO.getFieldType());
      dataItemMap.put(fieldPO.getFieldName().toLowerCase(), dataItemVO);
      MetadataEntityInfoItemPO infoItemPO = new MetadataEntityInfoItemPO();
      BeanUtils.copyProperties(fieldPO, infoItemPO);
      infoItemPO.setInfoItemName(fieldPO.getFieldCnName());
      infoItemPO.setDataItemName(fieldPO.getFieldName());
      syncInfoItems.add(infoItemPO);
    } 
  }
}
